<?php

/**
 * Back-end of Ipban_XH.
 *
 * Copyright (c) 2012 Christoph M. Becker (see license.txt)
 */


if (!defined('CMSIMPLE_XH_VERSION')) {
    header('HTTP/1.0 403 Forbidden');
    exit;
}


/*
 * Compatibility for PHP < 5.2
 */
if (!function_exists('json_encode')) {
    include_once $pth['folder']['plugins'] . 'ipban/json_encode.php';
    function json_encode($value)
    {
        return cmb_json_encode($value);
    }
}


/*
 * Compatibility for CMSimple_XH < 1.5.4
 */
if (!function_exists('XH_systemCheck')) {
    function XH_systemCheck($data)
    {
        global $pth, $plugin_tx;
        
        $ptx = $plugin_tx['ipban'];
        
        foreach (array('ok', 'warning', 'failure') as $img) {
            $txt = ucfirst($img);
            $imgs[$img] = tag('img src="' . $pth['folder']['plugins'] . 'ipban/images/' . $img . '.png" alt="'
                . $txt . '" title="' . $txt . '" width="16" height="16"');
        }
        
        $o = "<h4>$ptx[syscheck_title]</h4>\n<ul id=\"ipban_system_check\">\n";
        
        if (key_exists('phpversion', $data)) {
            $ok = version_compare(PHP_VERSION, $data['phpversion']) >= 0;
            $o .= '<li>' . $imgs[$ok ? 'ok' : 'fail']
                . sprintf($ptx['syscheck_phpversion'], $data['phpversion']) . "</li>\n";
        }
        
        if (key_exists('extensions', $data)) {
            $cat = ' class="ipban_system_check_cat_start"';
            foreach ($data['extensions'] as $ext) {
                if (is_array($ext)) {
                    $notok = $ext[1] ? 'failure' : 'warning';
                    $ext = $ext[0];
                } else {
                    $notok = 'failure';
                }
                $o .= '<li' . $cat . '>' . $imgs[extension_loaded($ext) ? 'ok' : $notok]
                    . sprintf($ptx['syscheck_extension'], $ext) . "</li>\n";
                $cat = '';
            }
        }
        
        if (key_exists('writable', $data)) {
            $cat = ' class="ipban_system_check_cat_start"';
            foreach ($data['writable'] as $file) {
                if (is_array($file)) {
                    $notok = $file[1] ? 'failure' : 'warning';
                    $file = $file[0];
                } else {
                    $notok = 'warning';
                }
                $o .= '<li' . $cat . '>' . $imgs[is_writable($file) ? 'ok' : $notok]
                    . sprintf($ptx['syscheck_writable'], $file) . "</li>\n";
                $cat = '';
            }
        }
        
        if (key_exists('other', $data)) {
            $cat = ' class="ipban_system_check_cat_start"';
            foreach ($data['other'] as $check) {
                $notok = $check[1] ? 'failure' : 'warning';
                $o .= '<li' . $cat . '>' . $imgs[$check[0] ? 'ok' : $notok]
                    . $check[2] . "</li>\n";
                $cat = '';
            }
        }
        
        $o .= "</ul>\n";
        
        return $o;
    }
}


/**
 * Saves the given $data in ban.dat.
 *
 * Returns whether that succeded.
 *
 * @param   array $data
 * @return  bool
 */
function IPBan_saveData($data)
{
    $fn = IPBan_dataFolder() . 'ban.dat';
    $succ = ($fp = fopen($fn, 'w')) !== false
        && fwrite($fp, serialize($data)) !== false;
    if ($fp !== false) {
        fclose($fp);
    }
    return $succ ? $succ : IPBan_error('cntsave', 'file', $fn);
}


/**
 * Compares two records. Helper for IPBan_sorted().
 *
 * @param   array $a
 * @param   array $b
 * @return  int
 */
function IPBan_compare($a, $b)
{
    list($fld) = explode(' ', $_GET['jtSorting']);
    switch ($fld) {
    case 'ip':
        return ip2long($a[$fld]) - ip2long($b[$fld]);
    case 'end':
    case 'time':
        return $a[$fld] - $b[$fld];
    default:
        return strcmp($a[$fld], $b[$fld]);
    }
}


/**
 * Returns the records sorted according to $_GET['jtsorting'].
 *
 * @param   array $data
 * @return  array
 */
function IPBan_sorted($data)
{
    uasort($data, 'IPBan_compare');
    list(, $dir) = explode(' ', $_GET['jtSorting']);
    if ($dir == 'DESC') {
        $data = array_reverse($data, true);
    }
    return $data;
}


/**
 * Returns an error response for jTable.
 *
 * @param   string $msg
 * @return  string
 */
function IPBan_jtError($msg)
{
    return json_encode(array('Result' => 'ERROR', 'Message' => $msg));
}


/**
 * Returns the record in a format suitable for the JSON response.
 */
function IPBan_responseRecord($id, $rec)
{
    return array('id' => $id,
                 'ip' => $rec['ip'],
                 'reason' => $rec['reason'],
                 'state' => $rec['warned'] ? true : false,
                 'end' => date('Y-m-d', $rec['end']));
}


/**
 * Returns the JSON response for the IP ban listing.
 *
 * @return  string
 */
function IPBan_listRecords()
{
    error_reporting(0);
    $data = IPBan_data();
    if (!is_array($data)) {
        return IPBan_jtError($data);
    }
    $data = IPBan_sorted($data);
    $trc = count($data);
    if (isset($_GET['jtPageSize'])) {
        $data = array_slice($data, $_GET['jtStartIndex'], $_GET['jtPageSize']);
    }
    $recs = array();
    foreach ($data as $id => $rec) {
        $recs[] = IPBan_responseRecord($id, $rec);
    }
    return json_encode(array('Result' => 'OK', 'Records' => $recs,
                             'TotalRecordCount' => $trc));
}


/**
 * Creates a new or updates an existing ban record.
 *
 * @param   bool $new
 * @return  string  The JSON response.
 */
function IPBan_postRecord($new)
{
    global $plugin_tx;
    
    error_reporting(0);
    $ptx = $plugin_tx['ipban'];
    $id = $new ? uniqid() : $_POST['id'];
    $rec = array('ip' => stsl($_POST['ip']),
                 'reason' => stsl($_POST['reason']),
                 'warned' => isset($_POST['state']),
                 'end' => strtotime($_POST['end']));
    if (long2ip(ip2long($rec['ip'])) != $rec['ip']) {
        return IPBan_jtError(sprintf($ptx['error_ip'], $rec['ip']));
    }
    if ($rec['end'] === false || $rec['end'] == -1) {
        return IPBan_jtError(sprintf($ptx['error_date'], $rec['date']));
    }
    $data = IPBan_data();
    if (!is_array($data)) {
        return IPBan_jtError($data);
    }
    if (($oldID = IPBan_find($rec['ip'], $data)) && $oldID != $id) {
        return IPBan_jtError(sprintf($ptx['error_duplicate'], $rec['ip']));
    }
    $data[$id] = $rec;
    if (($msg = IPBan_saveData($data)) !== true) {
        return IPBan_jtError($msg);
    }
    $res = array('Result' => 'OK');
    if ($new) {
        $res['Record'] = IPBan_responseRecord($id, $rec);
    }
    return json_encode($res);
}


/**
 * Deletes a ban record.
 *
 * @return  string  The JSON response.
 */
function IPBan_deleteRecord()
{
    error_reporting(0);
    $id = $_POST['id'];
    $data = IPBan_data();
    if (!is_array($data)) {
        return IPBan_jtError($data);
    }
    unset($data[$id]);
    if (($msg = IPBan_saveData($data)) != true) {
        return IPBan_jtError($msg);
    }
    return json_encode(array('Result' => 'OK'));
}


/**
 * Returns the POST requests.
 *
 * @return  array
 */
function IPBan_requests()
{
    $fn = IPBan_dataFolder() . 'post.log';
    if (($lines = file($fn)) === false) {
        return IPBan_error('cntopen', 'log', $fn);
    }
    $reqs = array();
    foreach ($lines as $line) {
        $flds = explode("\t", rtrim($line));
        $reqs[$flds[0]] = array('ip' => $flds[1],
                                'time' => $flds[2],
                                'vars' => $flds[3]);
    }
    return $reqs;
}


/**
 * Returns the filtered POST requests.
 *
 * @return  array
 */
function IPBan_filterRequests($reqs)
{
    if (!empty($_POST['var'])) {
        $fun = create_function(
            '$r',
            'return strpos($r[\'vars\'], ' . $_POST['var'] . ') !== false;');
        $reqs = array_filter($reqs, $fun);
    }
    return $reqs;
}


/**
 * Returns the JSON response for the POST request listing.
 *
 * @return  string
 */
function IPBan_listRequests()
{
    error_reporting(0);
    $reqs = IPBan_requests();
    if (!is_array($reqs)) {
        return IPBan_jtError($reqs);
    }
    $reqs = IPBan_sorted(IPBan_filterRequests($reqs));
    $trc = count($reqs);
    if (isset($_GET['jtPageSize'])) {
        $reqs = array_slice($reqs, $_GET['jtStartIndex'], $_GET['jtPageSize']);
    }
    $fun = create_function('&$r',
                           '$r["time"] = date("Y-m-d H:i:s", $r["time"]);');
    array_walk($reqs, $fun);               
    return json_encode(array('Result' => 'OK',
                             'Records' => $reqs,
                             'TotalRecordCount' => $trc));
}


/**
 * Returns the JSON response for the IP options request.
 *
 * @return  string
 */
function IPBan_ipOptions()
{
    error_reporting(0);
    $data = IPBan_data();
    if (!is_array($data)) {
        return IPBan_jtError($data);
    }
    $reqs = IPBan_requests();
    if (!is_array($reqs)) {
        return IPBan_jtError($reqs);
    }
    $ff = create_function('$rec', 'return $rec["ip"];');
    $ips = array_merge(array_map($ff, $data), array_map($ff, $reqs));
    $ips = array_unique($ips);
    usort($ips, create_function('$a, $b', 'return ip2long($a) - ip2long($b);'));
    $opts = array();
    foreach ($ips as $ip) {
        $opts[] = array('DisplayText' => $ip, 'Value' => $ip);
    }
    return json_encode(array('Result' => 'OK', 'Options' => $opts));
}


/**
 * Returns the version information view.
 *
 * @return string  The (X)HTML.
 */
function IPBan_version() {
    global $pth;

    return '<h1><a href="http://3-magi.net/?CMSimple_XH/Ipban_XH">Ipban_XH</a></h1>'
        . tag('img class="ipban_plugin_icon" src="' . $pth['folder']['plugins'] . 'ipban/ipban.png" alt="Plugin icon"')
        . '<p style="margin-top: 1em">Version: '.IPBAN_VERSION.'</p>'
        . '<p>Copyright &copy; 2012 <a href="http://3-magi.net/">Christoph M. Becker</a></p>'
        . '<p>The plugin icon is designed by <a href="http://VistaICO.com/">VistaICO.com</a>.</p>'
        . '<p class="ipban_license">This program is free software: you can redistribute it and/or modify'
        . ' it under the terms of the GNU General Public License as published by'
        . ' the Free Software Foundation, either version 3 of the License, or'
        . ' (at your option) any later version.</p>'
        . '<p class="ipban_license">This program is distributed in the hope that it will be useful,'
        . ' but WITHOUT ANY WARRANTY; without even the implied warranty of'
        . ' MERCHAN&shy;TABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the'
        . ' GNU General Public License for more details.</p>'
        . '<p class="ipban_license">You should have received a copy of the GNU General Public License'
        . ' along with this program.  If not, see'
        . ' <a href="http://www.gnu.org/licenses/">http://www.gnu.org/licenses/</a>.</p>';
}


/**
 * Returns the result view of the system check.
 *
 * @return string  The (X)HTML.
 */
function IPBan_systemCheck()
{
    global $pth, $tx, $plugin_tx;
    
    $ptx = $plugin_tx['ipban'];
    $data = array('phpversion' => '4.3.0',
                  'extensions' => array('date'),
                  'writable' => array(),
                  'other' => array());
    foreach (array('css', 'config', 'languages') as $dir) {
        $data['writable'][] = $pth['folder']['plugins'] . 'ipban/' . $dir;
    }
    $data['writable'][] = IPBan_dataFolder();
    sort($data['writable']);
    $data['other'][] = array(strtoupper($tx['meta']['codepage']) == 'UTF-8',
                             true, $ptx['syscheck_encoding']);
    $data['other'][] = array(!get_magic_quotes_runtime(), false,
                             $ptx['syscheck_magic_quotes']);
    $data['other'][] = array(file_exists($pth['folder']['plugins'] . 'jquery/jquery.inc.php'),
                             true, $ptx['syscheck_jquery']);
    return XH_systemCheck($data);
}


/**
 * Returns the main admin view.
 *
 * global   string $hjs
 * @return  string  The (X)HTML.
 */
function IPBan_admin()
{
    global $pth, $hjs, $sl, $cf, $plugin_cf, $plugin_tx;
    
    $pcf = $plugin_cf['ipban'];
    $ptx = $plugin_tx['ipban'];
    include_once $pth['folder']['plugins'] . 'jquery/jquery.inc.php';
    include_jQuery();
    include_jQueryUI();
    $hjs .= tag('link rel="stylesheet" href="' . $pth['folder']['plugins']
                . 'ipban/jtable/themes/lightcolor/blue/jtable.css" type="text/css"');
    include_jQueryPlugin('jtable', $pth['folder']['plugins']
                         . 'ipban/jtable/jquery.jtable.min.js');
    $lang = strlen($sl) == 2 ? $sl : $cf['language']['default'];
    if ($lang != 'en') {
        $fn = $pth['folder']['plugins']
            . 'ipban/languages/jquery.ui.datepicker-' . $lang . '.js';
        include_jQueryPlugin('DatepickerLang', $fn);
    }
    $hjs .= '<script type="text/javascript" src="' . $pth['folder']['plugins']
        . 'ipban/ipban.js"></script>' . "\n"
        . '<script type="text/javascript">/* <![CDATA[ */';
    foreach (array('jt' => 'messages', 'js' => 'texts') as $prefix => $var) {
        $msgs = array();
        foreach ($ptx as $key => $val) {
            list($pre, $key) = explode('_', $key);
            if ($pre == $prefix) {
                $msgs[] = $key . ':"' . addcslashes($val, "\t\r\n\\\"") . '"';
            }
        }
        $hjs .= 'ipban.' . $var . '={' . implode(',', $msgs) . '};';
    }
    $vars = array();
    foreach ($pcf as $key => $val) {
        list($pre, $key) = explode('_', $key);
        if ($pre == 'page') {
            $vars[] = $key . ':' . intval($val);
        }
    }
    $hjs .= 'ipban.page={' . implode(',', $vars) . '};ipban.lang=\'' . $lang
        . '\'/* ]]> */</script>';
    $o = '<div id="ipban_grid"></div>'
        . '<form id="ipban_filter_form">'
        . tag('input type="text" id="ipban_var"')
        . '<button type="submit" id="ipban_filter">' . $ptx['btn_filter'] . '</button>'
        . '</form>'
        . '<div id="ipban_requests"></div>';
    return $o;
}


/*
 * Handle the plugin main administration.
 */ 
if (isset($ipban) && $ipban == 'true') {
    $o .= print_plugin_admin('on');
    switch ($admin) {
    case '':
        $o .= IPBan_version() . tag('hr') . IPBan_systemCheck();
        break;
    case 'plugin_main':
        switch ($action) {
        case 'list': echo IPBan_listRecords(); exit;
        case 'create': echo IPBan_postRecord(true); exit;
        case 'update': echo IPBan_postRecord(false); exit;
        case 'delete': echo IPBan_deleteRecord(); exit;
        case 'requests': echo IPBan_listRequests(); exit;
        case 'ipoptions': echo IPBan_ipOptions(); exit;
        default: $o .= IPBan_admin();
        }
        break;
    default:
        $o .= plugin_admin_common($action, $admin, $plugin);
    }
}

?>
