<?php

/**
 * Front-end of Ipban_XH.
 *
 * Copyright (c) 2012 Christoph M. Becker (see license.txt)
 */


if (!defined('CMSIMPLE_XH_VERSION')) {
    header('HTTP/1.0 403 Forbidden');
    exit;
}


define('IPBAN_VERSION', '1beta2');


if (!defined('XH_ADM')) {
    define('XH_ADM', $adm);
}


/**
 * Returns an error message formatted in the style of e().
 *
 * @param   string $et  The error type.
 * @param   string $ft  The file type.
 * @param   string $fn  The file name.
 * @return  string
 */
function IPBan_error($et, $ft, $fn)
{
    global $tx;
    
    return '<b>' . $tx['error'][$et] . ' ' . $tx['filetype'][$ft] . '</b>'
        . tag('br') . $fn;
}


/**
 * Returns the data folder's path.
 *
 * @return  string
 */
function IPBan_dataFolder()
{
    global $pth, $plugin_cf;

    $pcf = $plugin_cf['ipban'];

    if ($pcf['folder_data'] == '') {
	$fn = $pth['folder']['plugins'].'ipban/data/';
    } else {
	$fn = $pth['folder']['base'].$pcf['folder_data'];
        if (substr($fn, -1) != '/') {
            $fn .= '/';
        }
    }
    return $fn;
}


/**
 * Returns the ban data.
 *
 * @return  array
 */
function IPBan_data()
{
    $fn = IPBan_dataFolder() . 'ban.dat';
    $cnt = is_readable($fn)
        and $cnt = file_get_contents($fn)
        and $cnt = unserialize($cnt);
    return $cnt !== false ? $cnt : IPBan_error('cntopen', 'file', $fn);
}


/**
 * Returns the ID of the ban record for the given $id.
 *
 * @param   string $ip
 * @param   array $data
 * @return  string
 */
function IPBan_find($ip, $data)
{
    $res = false;
    foreach ($data as $id => $rec) {
        if ($rec['ip'] == $ip) {
            $res = $id;
            break;
        }
    }
    return $res;
}


/**
 * Checks, if the current IP is banned, and handles this appropriately.
 *
 * @return  void
 */
function IPBan_ban()
{
    global $o, $e, $cf, $sl, $plugin_tx;
    
    $ptx = $plugin_tx['ipban'];
    $ip = XH_ADM && isset($_GET['IPBAN_DEBUG']) ? $_GET['IPBAN_DEBUG'] : $_SERVER['REMOTE_ADDR'];
    $data = IPBan_data();
    if (!is_array($data)) {
        $e .= '<li>' . $data . '</li>';
        return;
    }
    $id = IPBan_find($ip, $data);
    if ($id && time() < $data[$id]['end']) {
        $rec = $data[$id];
        $reason = $sl == $cf['language']['default'] && !empty($rec['reason'])
            ? $rec['reason']
            : $ptx['msg_reason'];
        $msg = strtr($ptx['msg_banned'],
                     array('{IP}' => $rec['ip'],
                           '{END}' => date($ptx['config_format_date'], $rec['end']),
                           '{REASON}' => $reason));
        if ($rec['warned']) {
            $o .= '<div id="ipban_warning"><div>' . $msg . '</div></div>';
        } else {
            header('HTTP/1.1 403 Forbidden');
            echo '<!DOCTYPE html><html><body>' . $msg . '</body></html>';
            exit;
        }
    }
}


/**
 * Logs a POST request.
 */
function IPBan_log()
{
    $entry = uniqid() . "\t" . $_SERVER['REMOTE_ADDR'] . "\t" . time() . "\t"
        . implode(",\xe2\x80\x8b", array_keys($_POST)) . "\n";
    $fn = IPBan_dataFolder() . 'post.log';
    if (($fp = fopen($fn, 'a')) === false || fwrite($fp, $entry) === false) {
        e('cntwriteto', 'log', $fn);
    }
    if ($fp !== false) {
        fclose($fp);
    }
}


/*
 * Log the request, if it's a POST from a visitor.
 */
if (!XH_ADM && !empty($_POST)) {
    IPBan_log();
}


/*
 * Handle a potential IP ban.
 */
IPBan_ban();

?>
